@@ -12,21 +12,32 @@ import (
 	"time"
 
 	"github.com/ghjm/cmdline"
+	"github.com/spf13/viper"
 )
 
-func InitCA(opts *CertOptions, certOut, keyOut string) error {
+// Oser is the function calls interfaces for mocking os.
+// type Oser interface {
+//	ReadFile(name string) ([]byte, error)
+//	WriteFile(name string, data []byte, perm fs.FileMode) error
+//}
+
+// OsWrapper is the Wrapper structure for Oser.
+// type OsWrapper struct{}
+
+// InitCA Initialize Certificate Authority.
+func InitCA(opts *CertOptions, certOut, keyOut string, osWrapper Oser) error {
 	ca, err := CreateCA(opts, &RsaWrapper{})
 	if err == nil {
-		err = SaveToPEMFile(certOut, []interface{}{ca.Certificate}, &OsWrapper{})
+		err = SaveToPEMFile(certOut, []interface{}{ca.Certificate}, osWrapper)
 	}
 	if err == nil {
-		err = SaveToPEMFile(keyOut, []interface{}{ca.PrivateKey}, &OsWrapper{})
+		err = SaveToPEMFile(keyOut, []interface{}{ca.PrivateKey}, osWrapper)
 	}
 
 	return err
 }
 
-type initCA struct {
+type InitCAConfig struct {
 	CommonName string `description:"Common name to assign to the certificate" required:"Yes"`
 	Bits       int    `description:"Bit length of the encryption keys of the certificate" required:"Yes"`
 	NotBefore  string `description:"Effective (NotBefore) date/time, in RFC3339 format"`
@@ -35,7 +46,7 @@ type initCA struct {
 	OutKey     string `description:"File to save the CA private key to" required:"Yes"`
 }
 
-func (ica initCA) Run() (err error) {
+func (ica InitCAConfig) Run() (err error) {
 	opts := &CertOptions{
 		CommonName: ica.CommonName,
 		Bits:       ica.Bits,
@@ -53,14 +64,15 @@ func (ica initCA) Run() (err error) {
 		}
 	}
 
-	return InitCA(opts, ica.OutCert, ica.OutKey)
+	return InitCA(opts, ica.OutCert, ica.OutKey, &OsWrapper{})
 }
 
-func MakeReq(opts *CertOptions, keyIn, keyOut, reqOut string) error {
+// MakeReq Create Certificate Request.
+func MakeReq(opts *CertOptions, keyIn, keyOut, reqOut string, osWrapper Oser) error {
 	var req *x509.CertificateRequest
 	var key *rsa.PrivateKey
 	if keyIn != "" {
-		data, err := LoadFromPEMFile(keyIn, &OsWrapper{})
+		data, err := LoadFromPEMFile(keyIn, osWrapper)
 		if err != nil {
 			return err
 		}
@@ -88,12 +100,12 @@ func MakeReq(opts *CertOptions, keyIn, keyOut, reqOut string) error {
 			return err
 		}
 	}
-	err := SaveToPEMFile(reqOut, []interface{}{req}, &OsWrapper{})
+	err := SaveToPEMFile(reqOut, []interface{}{req}, osWrapper)
 	if err != nil {
 		return err
 	}
 	if keyOut != "" {
-		err = SaveToPEMFile(keyOut, []interface{}{key}, &OsWrapper{})
+		err = SaveToPEMFile(keyOut, []interface{}{key}, osWrapper)
 		if err != nil {
 			return err
 		}
@@ -102,7 +114,7 @@ func MakeReq(opts *CertOptions, keyIn, keyOut, reqOut string) error {
 	return nil
 }
 
-type makeReq struct {
+type MakeReqConfig struct {
 	CommonName string   `description:"Common name to assign to the certificate" required:"Yes"`
 	Bits       int      `description:"Bit length of the encryption keys of the certificate"`
 	DNSName    []string `description:"DNS names to add to the certificate"`
@@ -113,7 +125,7 @@ type makeReq struct {
 	OutKey     string   `description:"File to save the private key to (new key will be generated)"`
 }
 
-func (mr makeReq) Prepare() error {
+func (mr MakeReqConfig) Prepare() error {
 	if mr.InKey == "" && mr.OutKey == "" {
 		return fmt.Errorf("must provide either InKey or OutKey")
 	}
@@ -130,7 +142,7 @@ func (mr makeReq) Prepare() error {
 	return nil
 }
 
-func (mr makeReq) Run() error {
+func (mr MakeReqConfig) Run() error {
 	opts := &CertOptions{
 		CommonName: mr.CommonName,
 		Bits:       mr.Bits,
@@ -148,22 +160,23 @@ func (mr makeReq) Run() error {
 		opts.IPAddresses = append(opts.IPAddresses, ip)
 	}
 
-	return MakeReq(opts, mr.InKey, mr.OutKey, mr.OutReq)
+	return MakeReq(opts, mr.InKey, mr.OutKey, mr.OutReq, &OsWrapper{})
 }
 
-func SignReq(opts *CertOptions, caCrtPath, caKeyPath, reqPath, certOut string, verify bool) error {
+// SignReq Sign Certificate Request.
+func SignReq(opts *CertOptions, caCrtPath, caKeyPath, reqPath, certOut string, verify bool, osWrapper Oser) error {
 	ca := &CA{}
 	var err error
-	ca.Certificate, err = LoadCertificate(caCrtPath, &OsWrapper{})
+	ca.Certificate, err = LoadCertificate(caCrtPath, osWrapper)
 	if err != nil {
 		return err
 	}
-	ca.PrivateKey, err = LoadPrivateKey(caKeyPath, &OsWrapper{})
+	ca.PrivateKey, err = LoadPrivateKey(caKeyPath, osWrapper)
 	if err != nil {
 		return err
 	}
 	var req *x509.CertificateRequest
-	req, err = LoadRequest(reqPath, &OsWrapper{})
+	req, err = LoadRequest(reqPath, osWrapper)
 	if err != nil {
 		return err
 	}
@@ -215,7 +228,7 @@ func SignReq(opts *CertOptions, caCrtPath, caKeyPath, reqPath, certOut string, v
 	return SaveToPEMFile(certOut, []interface{}{cert}, &OsWrapper{})
 }
 
-type signReq struct {
+type SignReqConfig struct {
 	Req       string `description:"Certificate Request PEM filename" required:"Yes"`
 	CACert    string `description:"CA certificate PEM filename" required:"Yes"`
 	CAKey     string `description:"CA private key PEM filename" required:"Yes"`
@@ -225,7 +238,7 @@ type signReq struct {
 	Verify    bool   `description:"If true, do not prompt the user for verification" default:"False"`
 }
 
-func (sr signReq) Run() error {
+func (sr SignReqConfig) Run() error {
 	opts := &CertOptions{}
 	if sr.NotBefore != "" {
 		t, err := time.Parse(time.RFC3339, sr.NotBefore)
@@ -242,14 +255,18 @@ func (sr signReq) Run() error {
 		opts.NotAfter = t
 	}
 
-	return SignReq(opts, sr.CACert, sr.CAKey, sr.Req, sr.OutCert, sr.Verify)
+	return SignReq(opts, sr.CACert, sr.CAKey, sr.Req, sr.OutCert, sr.Verify, &OsWrapper{})
 }
 
 func init() {
+	version := viper.GetInt("version")
+	if version > 1 {
+		return
+	}
 	cmdline.RegisterConfigTypeForApp("receptor-certificates",
-		"cert-init", "Initialize PKI CA", initCA{}, cmdline.Exclusive, cmdline.Section(certSection))
+		"cert-init", "Initialize PKI CA", InitCAConfig{}, cmdline.Exclusive, cmdline.Section(certSection))
 	cmdline.RegisterConfigTypeForApp("receptor-certificates",
-		"cert-makereq", "Create certificate request", makeReq{}, cmdline.Exclusive, cmdline.Section(certSection))
+		"cert-makereq", "Create certificate request", MakeReqConfig{}, cmdline.Exclusive, cmdline.Section(certSection))
 	cmdline.RegisterConfigTypeForApp("receptor-certificates",
-		"cert-signreq", "Sign request and produce certificate", signReq{}, cmdline.Exclusive, cmdline.Section(certSection))
+		"cert-signreq", "Sign request and produce certificate", SignReqConfig{}, cmdline.Exclusive, cmdline.Section(certSection))
 }
