@@ -4,6 +4,7 @@
 package controlsvc
 
 import (
+	"bufio"
 	"context"
 	"crypto/tls"
 	"encoding/json"
@@ -14,6 +15,7 @@ import (
 	"os"
 	"reflect"
 	"runtime"
+	"strconv"
 	"strings"
 	"sync"
 	"time"
@@ -22,6 +24,7 @@ import (
 	"github.com/ansible/receptor/pkg/netceptor"
 	"github.com/ansible/receptor/pkg/utils"
 	"github.com/ghjm/cmdline"
+	"github.com/spf13/viper"
 )
 
 const (
@@ -121,8 +124,38 @@ func (s *SockControl) ReadFromConn(message string, out io.Writer, io Copier) err
 	if err := s.WriteMessage(message); err != nil {
 		return err
 	}
-	if _, err := io.Copy(out, s.conn); err != nil {
-		return err
+	payloadDebug, _ := strconv.Atoi(os.Getenv("RECEPTOR_PAYLOAD_TRACE_LEVEL"))
+
+	if payloadDebug != 0 {
+		var connectionType string
+		var payload string
+		if s.conn.LocalAddr().Network() == "unix" {
+			connectionType = "unix socket"
+		} else {
+			connectionType = "network connection"
+		}
+		reader := bufio.NewReader(s.conn)
+
+		for {
+			response, err := reader.ReadString('\n')
+			if err != nil {
+				if err.Error() != "EOF" {
+					MainInstance.nc.GetLogger().Error("Error reading from conn: %v \n", err)
+				}
+
+				break
+			}
+			payload += response
+		}
+
+		MainInstance.nc.GetLogger().DebugPayload(payloadDebug, payload, "", connectionType)
+		if _, err := out.Write([]byte(payload)); err != nil {
+			return err
+		}
+	} else {
+		if _, err := io.Copy(out, s.conn); err != nil {
+			return err
+		}
 	}
 
 	return nil
@@ -353,7 +386,7 @@ func (s *Server) RunControlSession(conn net.Conn) {
 				}
 			}
 		} else {
-			writeMsg := "ERROR: Unknown command\n"
+			writeMsg := fmt.Sprintf("ERROR: Unknown command, %v\n", cmd)
 			if writeToConnWithLog(conn, s.nc, writeMsg, writeControlServiceError) {
 				return
 			}
@@ -489,7 +522,7 @@ type cmdlineConfigWindows struct {
 }
 
 // cmdlineConfigUnix is the cmdline configuration object for a control service on Unix.
-type cmdlineConfigUnix struct {
+type CmdlineConfigUnix struct {
 	Service     string `description:"Receptor service name to listen on" default:"control"`
 	Filename    string `description:"Specifies the filename of a local Unix socket to bind to the service."`
 	Permissions int    `description:"Socket file permissions" default:"0600"`
@@ -499,7 +532,7 @@ type cmdlineConfigUnix struct {
 }
 
 // Run runs the action.
-func (cfg cmdlineConfigUnix) Run() error {
+func (cfg CmdlineConfigUnix) Run() error {
 	if cfg.TLS != "" && cfg.TCPListen != "" && cfg.TCPTLS == "" {
 		netceptor.MainInstance.Logger.Warning("Control service %s has TLS configured on the Receptor listener but not the TCP listener.", cfg.Service)
 	}
@@ -515,7 +548,7 @@ func (cfg cmdlineConfigUnix) Run() error {
 		}
 	}
 	err = MainInstance.RunControlSvc(context.Background(), cfg.Service, tlscfg, cfg.Filename,
-		os.FileMode(cfg.Permissions), cfg.TCPListen, tcptls)
+		os.FileMode(cfg.Permissions), cfg.TCPListen, tcptls) //nolint:gosec
 	if err != nil {
 		return err
 	}
@@ -525,7 +558,7 @@ func (cfg cmdlineConfigUnix) Run() error {
 
 // Run runs the action.
 func (cfg cmdlineConfigWindows) Run() error {
-	return cmdlineConfigUnix{
+	return CmdlineConfigUnix{
 		Service:   cfg.Service,
 		TLS:       cfg.TLS,
 		TCPListen: cfg.TCPListen,
@@ -534,11 +567,15 @@ func (cfg cmdlineConfigWindows) Run() error {
 }
 
 func init() {
+	version := viper.GetInt("version")
+	if version > 1 {
+		return
+	}
 	if runtime.GOOS == "windows" {
 		cmdline.RegisterConfigTypeForApp("receptor-control-service",
 			"control-service", "Runs a control service", cmdlineConfigWindows{})
 	} else {
 		cmdline.RegisterConfigTypeForApp("receptor-control-service",
-			"control-service", "Runs a control service", cmdlineConfigUnix{})
+			"control-service", "Runs a control service", CmdlineConfigUnix{})
 	}
 }
