@@ -1,6 +1,3 @@
-//go:build !no_tls_config
-// +build !no_tls_config
-
 package netceptor
 
 import (
@@ -14,6 +11,7 @@ import (
 
 	"github.com/ansible/receptor/pkg/utils"
 	"github.com/ghjm/cmdline"
+	"github.com/spf13/viper"
 )
 
 // **************************************************************************
@@ -239,6 +237,10 @@ func (cfg TLSClientConfig) Prepare() error {
 }
 
 func init() {
+	version := viper.GetInt("version")
+	if version > 1 {
+		return
+	}
 	cmdline.RegisterConfigTypeForApp("receptor-tls",
 		"tls-server", "Define a TLS server configuration", TLSServerConfig{}, cmdline.Section(configSection))
 	cmdline.RegisterConfigTypeForApp("receptor-tls",
