@@ -1,3 +1,4 @@
+//go:generate mockgen -source=command.go -destination=mock_services/command.go
 //go:build !windows
 // +build !windows
 
@@ -5,54 +6,69 @@ package services
 
 import (
 	"crypto/tls"
+	"errors"
 	"net"
 	"os/exec"
 
 	"github.com/ansible/receptor/pkg/logger"
 	"github.com/ansible/receptor/pkg/netceptor"
-	"github.com/ansible/receptor/pkg/utils"
 	"github.com/creack/pty"
 	"github.com/ghjm/cmdline"
 	"github.com/google/shlex"
 	"github.com/spf13/viper"
 )
 
-func runCommand(qc net.Conn, command string, logger *logger.ReceptorLogger) error {
+type NetCForCommandService interface {
+	GetLogger() *logger.ReceptorLogger
+	ListenAndAdvertise(service string, tlscfg *tls.Config, tags map[string]string) (*netceptor.Listener, error)
+}
+
+func runCommand(qc net.Conn, command string, logger *logger.ReceptorLogger, utilsLib UtilsLib) error {
+	// Note: shlex.Split does not return error for the empty string
 	args, err := shlex.Split(command)
 	if err != nil {
 		return err
 	}
+	if len(args) == 0 {
+		return errors.New("shell command is empty")
+	}
 	cmd := exec.Command(args[0], args[1:]...)
 	tty, err := pty.Start(cmd)
 	if err != nil {
 		return err
 	}
-	utils.BridgeConns(tty, "external command", qc, "command service", logger)
+	utilsLib.BridgeConns(tty, "external command", qc, "command service", logger)
 
 	return nil
 }
 
 // CommandService listens on the Receptor network and runs a local command.
-func CommandService(s *netceptor.Netceptor, service string, tlscfg *tls.Config, command string) {
+func CommandService(s NetCForCommandService, service string, tlscfg *tls.Config, command string, utilsLib UtilsLib) {
+	if command == "" {
+		s.GetLogger().Error("initializing command service: command not provided\n")
+
+		return
+	}
+
 	qli, err := s.ListenAndAdvertise(service, tlscfg, map[string]string{
 		"type": "Command Service",
 	})
 	if err != nil {
-		s.Logger.Error("Error listening on Receptor network: %s\n", err)
+		s.GetLogger().Error("listening on Receptor network: %s\n", err)
 
 		return
 	}
 	for {
 		qc, err := qli.Accept()
 		if err != nil {
-			s.Logger.Error("Error accepting connection on Receptor network: %s\n", err)
+			s.GetLogger().Error("accepting connection on Receptor network: %s\n", err)
 
 			return
 		}
 		go func() {
-			err := runCommand(qc, command, s.Logger)
+			err := runCommand(qc, command, s.GetLogger(), utilsLib)
 			if err != nil {
-				s.Logger.Error("Error running command: %s\n", err)
+				s.GetLogger().Error("running command: %s\n", err)
 			}
 			_ = qc.Close()
 		}()
@@ -73,7 +89,8 @@ func (cfg CommandSvcCfg) Run() error {
 	if err != nil {
 		return err
 	}
-	go CommandService(netceptor.MainInstance, cfg.Service, tlscfg, cfg.Command)
+
+	go CommandService(netceptor.MainInstance, cfg.Service, tlscfg, cfg.Command, &UtilsTCPWrapper{})
 
 	return nil
 }
