@@ -4,20 +4,20 @@ import "fmt"
 
 // ErrorWithKind represents an error wrapped with a designation of what kind of error it is.
 type ErrorWithKind struct {
-	err  error
-	kind string
+	Err  error
+	Kind string
 }
 
 // Error returns the error text as a string.
 func (ek ErrorWithKind) Error() string {
-	return fmt.Sprintf("%s error: %s", ek.kind, ek.err)
+	return fmt.Sprintf("%s error: %v", ek.Kind, ek.Err)
 }
 
 // WrapErrorWithKind creates an ErrorWithKind that wraps an underlying error.
 func WrapErrorWithKind(err error, kind string) ErrorWithKind {
 	return ErrorWithKind{
-		err:  err,
-		kind: kind,
+		Err:  err,
+		Kind: kind,
 	}
 }
 
@@ -28,5 +28,5 @@ func ErrorIsKind(err error, kind string) bool {
 		return false
 	}
 
-	return ek.kind == kind
+	return ek.Kind == kind
 }
