@@ -198,6 +198,12 @@ func (rw *remoteUnit) startRemoteUnit(ctx context.Context, conn net.Conn, reader
 	if err != nil {
 		return fmt.Errorf("error opening stdin file: %s", err)
 	}
+	defer func() {
+		err := stdin.Close()
+		if err != nil {
+			MainInstance.nc.GetLogger().Error("Error closing %s: %s", path.Join(rw.UnitDir(), "stdin"), err)
+		}
+	}()
 	_, err = io.Copy(conn, stdin)
 	if err != nil {
 		return fmt.Errorf("error sending stdin file: %s", err)
@@ -467,6 +473,12 @@ func (rw *remoteUnit) monitorRemoteStdout(mw *utils.JobContext) {
 
 				return
 			}
+			defer func() {
+				err := stdout.Close()
+				if err != nil {
+					MainInstance.nc.GetLogger().Error("Error closing %s: %s", rw.StdoutFileName(), err)
+				}
+			}()
 			doneChan := make(chan struct{})
 			go func() {
 				select {
@@ -547,7 +559,6 @@ func (rw *remoteUnit) Status() *StatusFileData {
 // UnredactedStatus returns a copy of the status currently loaded in memory, including secrets.
 func (rw *remoteUnit) UnredactedStatus() *StatusFileData {
 	rw.GetStatusLock().RLock()
-	defer rw.GetStatusLock().RUnlock()
 	status := rw.GetStatusWithoutExtraData()
 	ed, ok := rw.GetStatusCopy().ExtraData.(*RemoteExtraData)
 	if ok {
@@ -558,6 +569,7 @@ func (rw *remoteUnit) UnredactedStatus() *StatusFileData {
 		}
 		status.ExtraData = &edCopy
 	}
+	rw.GetStatusLock().RUnlock()
 
 	return status
 }
